import torch 

class MeanIoUMetric:
    def __init__(self):
        self.epsilon = 1e-10
    def __call__(self, tensor1, tensor2):
        # if single dimension 
        if len(tensor1.shape) == 1 and len(tensor2.shape) == 1:
            inter = torch.sum(torch.squeeze(tensor1 * tensor2))
            union = torch.sum(torch.squeeze(tensor1 + tensor2)) - inter
        else:
            inter = torch.sum(torch.sum(torch.squeeze(tensor1 * tensor2, axis=3), axis=2), axis=1)
            union = torch.sum(torch.sum(torch.squeeze(tensor1 + tensor2, axis=3), axis=2), axis=1) - inter
        return torch.mean((inter + self.epsilon) / (union + self.epsilon))


# metrics = MeanIoUMetric()
# t1 = torch.tensor([0., 0., 1., 1.])
# t2 = torch.tensor([0., 1., 0., 1.])
# print(metrics(t1,t2))


import torch
from precision_recall import Precision, Recall

class F1Metric:
    def __init__(self):
        self.prec = Precision()
        self.recall = Recall()
        self.epsilon = 1e-10
    def __call__(self, tensor1, tensor2):
        precision = self.prec(tensor1, tensor2)
        recall = self.recall(tensor1, tensor2)
        return 2*((precision*recall)/(precision+recall+self.epsilon))



import torch

def corrcoef(tensor1, tensor2):
    """
    Arguments
    ---------
    x : torch.Tensor
    y : torch.Tensor
    """
    xm = tensor1.sub(torch.mean(tensor1))
    ym = tensor2.sub(torch.mean(tensor2))
    r_num = xm.dot(ym)
    r_den = torch.norm(xm, 2) * torch.norm(ym, 2)
    r_val = r_num / r_den
    return r_val

class RSquaredMetric:
    def __init__(self):
        self.corrcoef = corrcoef
    def __call__(self, tensor1, tensor2):
        """
        Arguments
        ---------
        x : torch.Tensor
        y : torch.Tensor
        """
        return (corrcoef(tensor1, tensor2))**2


import torch

class Precision:
    def __init__(self):
        self.epsilon = 1e-10
    def __call__(self, tensor1, tensor2):
        true_positives = torch.sum(torch.round(torch.clip(tensor1 * tensor2, 0, 1)))
        predicted_positives = torch.sum(torch.round(torch.clip(tensor2, 0, 1)))
        precision = true_positives / (predicted_positives + self.epsilon)
        return precision


class Recall:
    def __init__(self):
        self.epsilon = 1e-10
    def __call__(self, tensor1, tensor2):
        true_positives = torch.sum(torch.round(torch.clip(tensor1 * tensor2, 0, 1)))
        possible_positives = torch.sum(torch.round(torch.clip(tensor1, 0, 1)))
        recall = true_positives / (possible_positives + self.epsilon)
        return recall
    



import torch

class MAEMetric:
    def __call__(self, tensor1, tensor2):
        """
        Arguments
        ---------
        x : torch.Tensor
        y : torch.Tensor
        """
        return torch.mean(torch.abs(tensor1 - tensor2))



from setuptools import setup, find_packages

setup(
    name="torch_metrics",
    version="1.0.8",
    description="Metrics for model evaluation in pytorch",
    url="https://github.com/chinokenochkan/torch-metrics",
    author="Chi Nok Enoch Kan @chinokenochkan",
    author_email="kanxx030@gmail.com",
    packages=["torch_metrics"],
)


import torch

class DSCMetric:
    def __init__(self):
        self.smooth = 1.
    def __call__(self, tensor1, tensor2):
        iflat = tensor1.view(-1)
        tflat = tensor2.view(-1)
        intersection = (iflat * tflat).sum()
        
        return 1 - ((2. * intersection + self.smooth) /
                (iflat.sum() + tflat.sum() + self.smooth))


# metric = DSCMetric()
# t1 = torch.tensor([1., 0., 1., 0., 0.])
# t2 = torch.tensor([0., 0., 1., 1., 0.])
# print(metric(t1, t2))